@@ -6,7 +6,8 @@
 #
 # $Id$
 #
-# Copyright B. Cornec 2007
+# Copyright B. Cornec 2007-2012
+# Eric Anderson's changes are (c) Copyright 2012 Hewlett Packard
 # Provided under the GPL v2
 
 package ProjectBuilder::Filter;
@@ -19,6 +20,8 @@ use File::Copy;
 use lib qw (lib);
 use ProjectBuilder::Version;
 use ProjectBuilder::Base;
+use ProjectBuilder::Conf;
+use ProjectBuilder::Distribution;
 use ProjectBuilder::Changelog;
 
 # Inherit from the "Exporter" module which handles exporting functions.
@@ -61,53 +64,24 @@ The function returns a pointer on a hash of filters.
 sub pb_get_filters {
 
 my @ffiles;
-my ($ffile00, $ffile0, $ffile1, $ffile2, $ffile3, $ffile4, $ffile5);
-my ($mfile00, $mfile0, $mfile1, $mfile2, $mfile3, $mfile4, $mfile5);
 my $pbpkg = shift || die "No package specified";
 my $pbos = shift;
 my $ptr = undef; # returned value pointer on the hash of filters
 my %h;
 
 pb_log(2,"Entering pb_get_filters - pbpkg: $pbpkg - pbos: ".Dumper($pbos)."\n");
-# Global filter files first, then package specificities
-if (-d "$ENV{'PBROOTDIR'}/pbfilter") {
-	$mfile00 = "$ENV{'PBROOTDIR'}/pbfilter/all.pbf" if (-f "$ENV{'PBROOTDIR'}/pbfilter/all.pbf");
-	if (defined $pbos) {
-		$mfile0 = "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'os'}.pbf" if ((defined $pbos->{'os'}) && (-f "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'os'}.pbf"));
-		$mfile1 = "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'type'}.pbf" if ((defined $pbos->{'type'}) && (-f "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'type'}.pbf"));
-		$mfile2 = "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'family'}.pbf" if ((defined $pbos->{'family'}) && (-f "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'family'}.pbf"));
-		$mfile3 = "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'name'}.pbf" if ((defined $pbos->{'name'}) && (-f "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'name'}.pbf"));
-		$mfile4 = "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'name'}-$pbos->{'version'}.pbf" if ((defined $pbos->{'name'}) && (defined $pbos->{'version'}) && (-f "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'name'}-$pbos->{'version'}.pbf"));
-		$mfile5 = "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'name'}-$pbos->{'version'}-$pbos->{'arch'}.pbf" if ((defined $pbos->{'name'}) && (defined $pbos->{'version'}) && (defined $pbos->{'arch'}) && (-f "$ENV{'PBROOTDIR'}/pbfilter/$pbos->{'name'}-$pbos->{'version'}-$pbos->{'arch'}.pbf"));
-	}
 
-	push @ffiles,$mfile00 if (defined $mfile00);
-	push @ffiles,$mfile0 if (defined $mfile0);
-	push @ffiles,$mfile1 if (defined $mfile1);
-	push @ffiles,$mfile2 if (defined $mfile2);
-	push @ffiles,$mfile3 if (defined $mfile3);
-	push @ffiles,$mfile4 if (defined $mfile4);
-	push @ffiles,$mfile5 if (defined $mfile5);
+# Global filter files first, then package specific
+my @file_basenames = ('all');
+@file_basenames = reverse pb_distro_to_keylist($pbos, 'all') if (defined $pbos);
+# Build list of all filter files
+foreach my $dir ("$ENV{PBROOTDIR}/pbfilter", "$ENV{PBROOTDIR}/$pbpkg/pbfilter") {
+	foreach my $file_basename (@file_basenames) {
+		my $path = "$dir/${file_basename}.pbf";
+		push(@ffiles, $path) if -f $path;
+	}
 }
 
-if (-d "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter") {
-	$ffile00 = "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/all.pbf" if (-f "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/all.pbf");
-	if (defined $pbos) {
-		$ffile0 = "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'os'}.pbf" if ((defined $pbos->{'os'}) && (-f "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'os'}.pbf"));
-		$ffile1 = "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'type'}.pbf" if ((defined $pbos->{'type'}) && (-f "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'type'}.pbf"));
-		$ffile2 = "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'family'}.pbf" if ((defined $pbos->{'family'}) && (-f "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'family'}.pbf"));
-		$ffile3 = "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'name'}.pbf" if ((defined $pbos->{'name'}) && (-f "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'name'}.pbf"));
-		$ffile4 = "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'name'}-$pbos->{'version'}.pbf" if ((defined $pbos->{'name'}) && (defined $pbos->{'version'}) && (-f "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'name'}-$pbos->{'version'}.pbf"));
-		$ffile5 = "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'name'}-$pbos->{'version'}-$pbos->{'arch'}.pbf" if ((defined $pbos->{'name'}) && (defined $pbos->{'version'}) && (defined $pbos->{'arch'}) && (-f "$ENV{'PBROOTDIR'}/$pbpkg/pbfilter/$pbos->{'name'}-$pbos->{'version'}-$pbos->{'arch'}.pbf"));
-	}
-	push @ffiles,$ffile00 if (defined $ffile00);
-	push @ffiles,$ffile0 if (defined $ffile0);
-	push @ffiles,$ffile1 if (defined $ffile1);
-	push @ffiles,$ffile2 if (defined $ffile2);
-	push @ffiles,$ffile3 if (defined $ffile3);
-	push @ffiles,$ffile4 if (defined $ffile4);
-	push @ffiles,$ffile5 if (defined $ffile5);
-}
 if (@ffiles) {
 	pb_log(2,"DEBUG ffiles: ".Dumper(\@ffiles)."\n");
 
@@ -116,13 +90,40 @@ if (@ffiles) {
 		open(CONF,$f) || next;
 		while(<CONF>)  {
 			if (/^\s*([A-z0-9-_]+)\s+([[A-z0-9-_]+)\s*=\s*(.+)$/) {
-				pb_log(3,"DEBUG creating entry $1, key $2, value $3\n");
-				$h{$1}{$2}=$3;
+				my ($what, $var, $value) = ($1, $2, $3);
+				pb_log(3,"DEBUG creating entry $what, key $var, value $value\n");
+				# Add support for multi-lines
+				while ($value =~ s/\\\s*$//o) {
+					$_ = <CONF>;
+					die "Still processing continuations for $what $var at EOF" if (not defined $_);
+					s/[\r\n]//go;
+					$value .= "\n$_";
+				}
+				$h{$what}{$var}=$value;
+			} elsif ((/^\s*#/o) || (/^\s*$/o)) {
+				# ignore
+			} else {
+				chomp();
+				warn "unexpected line '$_' in $f";
 			}
 		}
 		close(CONF);
 	}
 	$ptr = $h{"filter"};
+
+	# TODO: find a place to document it. Why not in this file as pod and also documenting filters ?
+	# Handle transform
+	if (defined $h{transform}) {
+		while (my ($out_key,$spec) = each %{$h{transform}}) {
+			die "Unknown transform for $out_key '$spec' expected <out-key> <transform>" unless $spec =~ /^([\w\-]+)\s+(.+)$/;
+			my ($in_key, $expr) = ($1, $2);
+			local $_ = $ptr->{$in_key} || '';
+			eval $expr;
+			die "Error evaluating tranform for $out_key ($expr): $@" if $@;
+			$ptr->{$out_key} = $_;
+			pb_log(2, "Transform $in_key to $out_key\n$ptr->{$in_key}\n$ptr->{$out_key}\n");
+		}
+	}
 }
 pb_log(2,"DEBUG f:".Dumper($ptr)."\n") if (defined $ptr);
 return($ptr);
@@ -173,6 +174,7 @@ while (<FILE>) {
 			pb_log(3,"*** Filtering variable in $tmp ***\n");
 			# Order is important as we need to handle hashes refs before simple vars
 			# (?: introduce a Non-capturing groupings cf man perlretut
+			# We need to avoid handling other VARs (Makefile e.g) so restrict here to $pb type of vars.
 			eval { $tmp =~ s/(\$\w+(?:-\>\{\'\w+\'\})*)/$1/eeg };
 			if (($s =~ /^PBDESC$/) && ($line =~ /^ PBDESC/)) {
 				# if on debian, we need to preserve the space before each desc line
@@ -180,6 +182,7 @@ while (<FILE>) {
 				$tmp =~ s/\$\//\$\/ /g;
 				pb_log(3,"*** tmp:$tmp ***\n");
 			}
+			# Support $/ vars
 			eval { $tmp =~ s/(\$\/)/$1/eeg };
 		} elsif (($s =~ /^PBLOG$/) && ($line =~ /^PBLOG$/)) {
 			# special case for ChangeLog only for pb
@@ -214,9 +217,10 @@ while (<FILE>) {
 			pb_log(3,"DEBUG($tuple) filtering PBPATCHCMD\n");
 			my $i = 0;
 			if (defined $pb->{'patches'}->{$tuple}) {
+				my ($patchcmd,$patchopt) = pb_distro_get_param($pb->{'pbos'},pb_conf_get_if("ospatchcmd","ospatchopt"));
 				foreach my $p (split(/,/,$pb->{'patches'}->{$tuple})) {
 					pb_log(3,"DEBUG($tuple) Adding patch command $i\n");
-					print DEST "%patch$i -p1\n";
+					print DEST "%patch$i $patchopt\n";
 					$i++;
 				}
 			}
